-- Set the caption
love.graphics.setCaption("Instance Demo")

-- List to store objects
local objectList = {}

-- Deleted object placeholder
local deleted = {order = math.huge}

-- Sorts the object list and removes deleted objects
function objectList.sort()
    table.sort(objectList, function(a,b) return a.order < b.order end)
    local last = #objectList
    while objectList[last] == deleted do
        objectList[last] = nil
        last = last - 1
    end
end

-- Circle class.
local Circle = {}
Circle.__index = Circle

-- Create and return a new circle
function Circle:new(x, y)
    local circle = setmetatable({}, Circle)
    circle.direction = math.random() * 2 * math.pi
    circle.speed = 50 + math.random() * 100
    circle.size = 5 + math.random() * 15
    circle.life = 0.5 + math.random() * 2
    circle.color = {255*math.random(), 255*math.random(), 255*math.random(), 255}
    circle.x = x or 0
    circle.y = y or 0
    circle.order = math.random()
    return circle
end

-- Update the circle
function Circle:update(dt)
    self.life = self.life - dt
    if self.life < 0 then self:delete() end
    self.x = self.x + math.cos(self.direction) * self.speed * dt
    self.y = self.y - math.sin(self.direction) * self.speed * dt
end

-- Draw the circle
function Circle:draw()
    love.graphics.setColor(self.color)
    love.graphics.circle("fill", self.x, self.y, self.size)
    love.graphics.setColor(255,255,255,255)
end

-- Delete the circle
function Circle:delete()
    for i = 1,#objectList do
        if objectList[i] == self then
            objectList[i] = deleted
        end
    end
end

-- On mousepressed create a 10 new circles
function love.mousepressed(x,y,mb)
    for i = 1,10 do
        objectList[#objectList+1] = Circle:new(x,y)
    end
end

-- Update everything in the object list and then sort it
function love.update(dt)
    for i = 1,#objectList do
        if objectList[i].update then objectList[i]:update(dt) end
    end
    objectList.sort()
end

-- Draw everything in the object list
function love.draw()
    for i = 1,#objectList do
        if objectList[i].draw then objectList[i]:draw() end
    end
    love.graphics.print("Click to create circles!",0,0)
end